"""
MIT License

Copyright (c) 2023 Wilhelm Ågren

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

File created: 2023-05-08
Last updated: 2023-05-28
"""

from .info import *
from .utils import *
from .base import *

from synd.models import *
from synd.lineage import *
from synd.datautil import *
from synd.datasets import *

# Create logger and set up configuration for console as stdout
# Levels in decreasing order of verbosity:
#   - NOTSET     0
#   - DEBUG     10
#   - INFO      20
#   - WARNING   30
#   - ERROR     40
#   - CRITICAL  50
#
# To change the logging level after having imported the library,
# use the function `set_logging_level(level)` with preferred logging level.

import logging
log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)

console_handler = logging.StreamHandler()
console_handler.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    '[%(asctime)s] [%(name)s] [%(levelname)s\t] %(message)s'
)

console_handler.setFormatter(formatter)
log.addHandler(console_handler)

import builtins
import numpy as np
import torch

Integer = builtins.int

def set_log_level(level: Integer):
    log.setLevel(level)

def set_rng_seed(seed: Integer):
    np.random.seed(seed)
    torch.manual_seed(seed)

set_rng_seed(192881741)

