"""
MIT License

Copyright (c) 2023 Wilhelm Ågren

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

File created: 2023-05-28
Last updated: 2023-05-28
"""

import logging
from abc import ABC

from synd.utils import create_timestamp
from synd.typing import *

log = logging.getLogger(__name__)

class Base(ABC):
    """ """

    def __init__(self, *args: Tuple[Any, ...], **kwargs: Dict):
        name = kwargs.get('name', None)

        if name is None:
            name = self.__class__.__name__ + create_timestamp()

        log.debug(f'setting `{self.__class__.__name__}` name to: `{name}`')
        self._name = name

    def __repr__(self) -> String:
        return f'<{self.__class__.__name__} at {hex(id(self))}>'

    def __str__(self) -> String:
        return self._name

