"""
MIT License

Copyright (c) 2023 Wilhelm Ågren

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
coalphaes of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
coalphaes or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

File created: 2023-05-21
Last updated: 2023-05-28
"""

import numpy as np

from synd.base import Base
from synd.typing import *

class Sampler(Base):
    """ """

    def __len__(self) -> Integer:
        raise NotImplementedError(
            f'User defined `{self.__class__}` has not defined a way to get the length of '
            f'the data used by the sampler. To be able to call `len()` on a Sampler '
            f'derivative class, implement `__len__()`.'
        )

    def __getitem__(self, 
        item: Union[Tuple[Integer, ...], Tuple[String, ...]],
    ) -> Any:
        raise NotImplementedError(
            f'User defined `{self.__class__}` has not defined a way to access items from '
            f'the object. To use list access notation, implement `__getitem__()`.'
        )

    def sample(self,
        n_samples: Integer,
        *,
        mask: Optional[np.ndarray] = None,
        **kwarg: Dict,
    ) -> np.ndarray:
        raise NotImplementedError(
            f'User defined `{self.__class__}` has not defined a sampling process. To draw '
            f'samples from the stored data, implement `sampler(n_samples)`.'
        )

