"""
MIT License

Copyright (c) 2023 Wilhelm Ågren

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

File created: 2023-05-21
Last updated: 2023-05-28
"""

from synd.base import Base
from synd.typing import *

class Transformer(Base):
    """ """

    def __init__(self, *args, **kwargs):
        super(Transformer, self).__init__(*args, **kwargs)

        self._fit_fn = {
            'discrete': self._fit_discrete,
            'continuous': self._fit_continuous,
        }

        self._transform_fn = {
            'discrete': self._transform_discrete,
            'continuous': self._transform_continuous,
        }

        self._inverse_transform_fn = {
            'discrete': self._inverse_transform_discrete,
            'continuous': self._inverse_transform_continuous,
        }

    def _fit_discrete(self, *args: Tuple[Any, ...], **kwargs: Dict):
        raise NotImplementedError(
            f'`{self}` has no way to fit discrete data, please implement this function.'
        )

    def _fit_continuous(self, *args: Tuple[Any, ...], **kwargs: Dict):
        raise NotImplementedError(
            f'`{self}` has no way to fit continuous data, please implement this function.'
        )

    def _transform_discrete(self, *args: Tuple[Any, ...], **kwargs: Dict):
        raise NotImplementedError(
            f'`{self}` has no way to transform discrete data, '
            f'please implement this function.'
        )

    def _transform_continuous(self, *args: Tuple[Any, ...], **kwargs: Dict):
        raise NotImplementedError(
            f'`{self}` has no way to transform continuous data, '
            f'please implement this function.'
        )

    def _inverse_transform_discrete(self, *args: Tuple[Any, ...], **kwargs: Dict):
        raise NotImplementedError(
            f'`{self}` has no way to inverse transform discrete data, '
            f'please implement this function.'
        )

    def _inverse_transform_continuous(self, *args: Tuple[Any, ...], **kwargs: Dict):
        raise NotImplementedError(
            f'`{self}` has no way to inverse transform continuous data, '
            f'please implement this function.'
        )

    def is_fitted(self) -> Boolean:
        return hasattr(self, '_fitted')

    def assert_is_fitted(self):
        if not self.is_fitted():
            raise ValueError(
                f'`{self}` has not been fitted, please run `.fit()` with your '
                f'dataset before attempting either transform or inverse_transform.'
            )

    def fit(self, *args: Tuple[Any, ...], **kwargs: Dict):
        """ """
        raise NotImplementedError(
            f'No fit method is implemented for `{self.__class__.__name__}`. '
            f'Please implement a data fitting process for the transformer.'
        )

    def transform(self, *args: Tuple[Any, ...], **kwargs: Dict):
        raise NotImplementedError(
            f'No transform method is implemented for `{self.__class__.__name__}`. '
            f'Please implement a way to transform data with the fitted transformer.'
        )

    def inverse_transform(self, *args: Tuple[Any, ...], **kwargs: Dict):
        raise NotImplementedError(
            f'No inverse transform method is implemented for `{self.__class__.__name__}`. '
            f'Please implement a process for inverting transformed data to original form.'
        )

