"""
MIT License

Copyright (c) 2023 Wilhelm Ågren

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

File created: 2023-05-11
Last updated: 2023-05-28
"""

import contextlib
import numpy as np
import pandas as pd
import torch

import pytz as timezone
from datetime import datetime
from functools import wraps

from synd.info import ActivationFnInfo
from synd.typing import *

def is_discrete_column(col_info: List[ActivationFnInfo]) -> Boolean:
    return all([col._activation_fn == 'softmax' for col in col_info])

def create_timestamp(strformat: String = '::%Y-%m-%d::%H:%M:%S') -> String:
    return datetime.now(timezone.utc).strftime(strformat)

def get_rng_state() -> Dict:
    return {'numpy': np.random.get_state(), 'torch': torch.get_rng_state()}


